@@ -78,6 +78,8 @@ impl<T> Node<T> {
         ))
     }
 
+    // TODO: select -> success -> reset
+
     pub fn named_select(name: &str, nodes: Vec<Node<T>>) -> Node<T> {
         Self::new_named(
             name.to_owned(),
@@ -110,24 +112,32 @@ impl<T> Node<T> {
         })
     }
 
-    pub fn named_while_single(name: &str, cond: fn(&T) -> bool, child: Node<T>) -> Node<T> {
+    pub fn named_while_single(name: &str, cond: Box<dyn Fn(&T) -> bool>, child: Node<T>) -> Node<T> {
         Self::new_named(
             name.to_owned(),
             Behavior::While(cond, Rc::new(RefCell::new(child))),
         )
     }
 
-    pub fn while_single(cond: fn(&T) -> bool, child: Node<T>) -> Node<T> {
-        Self::new(Behavior::While(cond, Rc::new(RefCell::new(child))))
+    pub fn named_while_single_child(name: &str, cond: Box<dyn Fn(&T) -> bool>, child: Rc<RefCell<Node<T>>>) -> Node<T> {
+        Self::new_named(
+            name.to_owned(),
+            Behavior::While(cond, child),
+        )
     }
 
-    pub fn collapse(self, _desc: &str) -> Node<T> {
-        self
+
+    // pub fn while_single(cond: fn(&T) -> bool, child: Node<T>) -> Node<T> {
+    //     Self::new(Behavior::While(cond, Rc::new(RefCell::new(child))))
+    // }
+
+    pub fn collapse(self, desc: &str) -> Node<T> {
+        // self
         // TODO: re-enable once behavior-tree-egui catches up
-        // Self {
-        //     collapse_as: Some(desc.to_owned()),
-        //     ..self
-        // }
+        Self {
+            collapse_as: Some(desc.to_owned()),
+            ..self
+        }
     }
 
     pub fn reset(&mut self) {
@@ -149,13 +159,42 @@ impl<T> Node<T> {
         match &self.behavior {
             Behavior::Wait { .. } => vec![],
             Behavior::Cond(_, _, positive, negative) => vec![positive.clone(), negative.clone()],
-            _ => vec![],
             Behavior::Sequence(_, ref seq) => seq.clone(),
-            Behavior::Select(_, _) => todo!(),
-            Behavior::Action(_, _) => todo!(),
-            Behavior::ActionSuccess(_, _) => todo!(),
-            Behavior::StatefulAction(_, _) => todo!(),
-            Behavior::While(_, _) => todo!(),
+            Behavior::Select(_, ref seq) => seq.clone(),
+            Behavior::Action(_, _) => vec![],
+            Behavior::ActionSuccess(_, _) => vec![],
+            Behavior::StatefulAction(_, _) => vec![],
+            Behavior::While(_, item) => vec![item.clone()],
+        }
+    }
+
+    pub fn name(&self) -> String {
+        match &self.collapse_as {
+            Some(collapse_text) => collapse_text.clone(),
+            None => {
+                match &self.behavior {
+                    Behavior::Wait { curr, max } => format!("Wait {:.2}/{:.2}", curr, max),
+                    Behavior::Cond(name, _cond, _a, _b) => format!("Cond {}", name),
+                    // TreeRepr::new("Cond", vec![a.borrow().to_debug(), b.borrow().to_debug()])
+                    //     .with_detail(name.clone())
+                    Behavior::Sequence(_, _seq) => "Sequence".to_string(),
+                    // if let Some(ref name) = self.name {
+                    //     format!("Sequence {}", name)
+                    // } else {
+                    //     "Sequence".to_string()
+                    // },
+                    // seq.iter().map(|x| x.borrow().to_debug()).collect(),
+                    Behavior::Select(_, _seq) => "Select".to_string(),
+                    Behavior::Action(name, _) => format!("Action {}", name),
+                    Behavior::ActionSuccess(name, _) => format!("ActionSuccess {}", name),
+                    Behavior::StatefulAction(name, _) => format!("StatefulAction {}", name),
+                    // Behavior::While(_, x) => TreeRepr::new("While", vec![x.to_debug()]),
+                    // TODO: add to detail
+                    Behavior::While(_, _x) => {
+                        format!("While {}", self.name.as_ref().expect("While must have a name"))
+                    }
+                }
+            }
         }
     }
 
@@ -205,4 +244,20 @@ impl<T> Node<T> {
         repr.status = self.status;
         repr
     }
+
+    pub fn recheck_condition(&mut self, context: &T, is_sequence: bool) -> bool {
+        match &self.behavior {
+            Behavior::While(cond, _) => {
+                cond(context) != is_sequence
+            }
+            _ => false,
+            // Behavior::Sequence(_, _) => todo!(),
+            // Behavior::Select(_, _) => todo!(),
+            // Behavior::Action(_, _) => todo!(),
+            // Behavior::ActionSuccess(_, _) => todo!(),
+            // Behavior::StatefulAction(_, _) => todo!(),
+            // Behavior::Wait { curr, max } => todo!(),
+            // Behavior::Cond(_, _, _, _) => todo!(),
+        }
+    }
 }
